@@ -101,7 +101,6 @@ import os
 import shutil
 import sys
 
-from anytree import Node, PreOrderIter
 from copy import deepcopy
 from glob import glob
 from lxml import etree
@@ -254,6 +253,57 @@ def get_chunk_titles(module, node, tree_node):
     return result
 
 
+class PreOrderIter:
+    def __init__(self, node):
+        self.__node = node
+
+    def __iter__(self):
+        yield self.__node
+
+        for child in self.__node.descendants:
+            yield child
+
+
+class Node:
+    def __init__(self, name, parent=None, **kwargs):
+        self.name = name
+        self.__root = None
+        self.__attrs = kwargs
+        self.children = []
+
+        assert parent is None or isinstance(parent, Node)
+        self.parent = parent
+        if parent is not None:
+            self.__root = parent.root
+            parent.children.append(self)
+
+    @property
+    def root(self):
+        return self.__root or self
+
+    @property
+    def descendants(self):
+        ret = []
+
+        for child in self.children:
+            ret.append(child)
+
+            for other in child.descendants:
+                ret.append(other)
+
+        return ret
+
+    def __iter__(self):
+        for child in self.children:
+            yield child
+
+    def __getattr__(self, name):
+        try:
+            return self.__attrs[name]
+        except KeyError as e:
+            raise AttributeError(str(e))
+
+
 def chunk(xml_node, module, depth=0, idx=0, parent=None):
     """Chunk the tree.
 
@@ -1751,19 +1801,19 @@ def create_devhelp2(out_dir, module, xml, files):
             idx.write(line)
 
 
-def get_dirs(uninstalled):
-    if uninstalled:
-        # this does not work from buiddir!=srcdir
-        gtkdocdir = os.path.split(sys.argv[0])[0]
-        if not os.path.exists(gtkdocdir + '/gtk-doc.xsl'):
-            # try 'srcdir' (set from makefiles) too
-            if os.path.exists(os.environ.get("ABS_TOP_SRCDIR", '') + '/gtk-doc.xsl'):
-                gtkdocdir = os.environ['ABS_TOP_SRCDIR']
-        styledir = gtkdocdir + '/style'
-    else:
-        gtkdocdir = os.path.join(config.datadir, 'gtk-doc/data')
-        styledir = gtkdocdir
-    return (gtkdocdir, styledir)
+class FakeDTDResolver(etree.Resolver):
+    """Don't load the docbookx.dtd since we disable the validation anyway.
+
+    libxsml2 does not cache DTDs. If we produce a docbook file with 100 chunks
+    loading such a doc with xincluding will load and parse the docbook DTD 100
+    times. This cases tons of memory allocations and is slow.
+    """
+
+    def resolve(self, url, id, context):
+        if not url.endswith('.dtd'):
+            return None
+        # http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd
+        return self.resolve_string('', context)
 
 
 def main(module, index_file, out_dir, uninstalled, src_lang, paths):
@@ -1773,10 +1823,9 @@ def main(module, index_file, out_dir, uninstalled, src_lang, paths):
 
     # 1) load the docuemnt
     _t = timer()
-    # does not seem to be faster
-    # parser = etree.XMLParser(dtd_validation=False, collect_ids=False)
-    # tree = etree.parse(index_file, parser)
-    tree = etree.parse(index_file)
+    parser = etree.XMLParser(dtd_validation=False, collect_ids=False)
+    parser.resolvers.add(FakeDTDResolver())
+    tree = etree.parse(index_file, parser)
     logging.warning("1a: %7.3lf: load doc", timer() - _t)
     _t = timer()
     tree.xinclude()
@@ -1785,7 +1834,7 @@ def main(module, index_file, out_dir, uninstalled, src_lang, paths):
     # 2) copy datafiles
     _t = timer()
     # TODO: handle additional images
-    (gtkdocdir, styledir) = get_dirs(uninstalled)
+    (gtkdocdir, styledir) = config.get_dirs(uninstalled)
     # copy navigation images and stylesheets to html directory ...
     css_file = os.path.join(styledir, 'style.css')
     for f in glob(os.path.join(styledir, '*.png')) + [css_file]:
@@ -1863,6 +1912,10 @@ def run(options):
     module = options.args[0]
     document = options.args[1]
 
+    output_dir = options.output_dir or os.getcwd()
+    if options.output_dir and not os.path.isdir(output_dir):
+        os.mkdir(output_dir)
+
     # TODO: pass options.extra_dir
-    sys.exit(main(module, document, os.getcwd(), options.uninstalled, options.src_lang,
+    sys.exit(main(module, document, output_dir, options.uninstalled, options.src_lang,
                   options.path))
