@@ -150,6 +150,20 @@ annotation_outparams_optional (GList **list)
   return TRUE;
 }
 
+/**
+ * annotation_outparams_not_optional:
+ * @list: (out) (transfer none) (not optional): a pointer to take a list
+ *
+ * Document optional parameters.
+ *
+ * Returns: %TRUE for success
+ */
+gboolean
+annotation_outparams_not_optional (GList **list)
+{
+  return TRUE;
+}
+
 /**
  * annotation_outparams_nullable:
  * @list: (out) (transfer none) (nullable): a pointer to take a list; but %NULL
@@ -195,6 +209,21 @@ annotation_outparams_optional_nullable (GList **list)
   return TRUE;
 }
 
+/**
+ * annotation_outparams_not_optional_nullable:
+ * @list: (out) (transfer none) (not optional) (nullable): a pointer to take a
+ * list, not %NULL; but %NULL may also be returned in @list — isn’t that cool?
+ *
+ * Document non-optional parameters.
+ *
+ * Returns: %TRUE for success
+ */
+gboolean
+annotation_outparams_not_optional_nullable (GList **list)
+{
+  return TRUE;
+}
+
 /**
  * annotation_outparams_optional_not_nullable:
  * @list: (out) (transfer none) (optional) (not nullable): a pointer to take a
@@ -210,6 +239,21 @@ annotation_outparams_optional_not_nullable (GList **list)
   return TRUE;
 }
 
+/**
+ * annotation_outparams_not_optional_not_nullable:
+ * @list: (out) (transfer none) (not optional) (not nullable): a pointer to take a
+ * list, not %NULL; and %NULL must not be returned in @list — isn’t that cool?
+ *
+ * Document non-optional parameters.
+ *
+ * Returns: %TRUE for success
+ */
+gboolean
+annotation_outparams_not_optional_not_nullable (GList **list)
+{
+  return TRUE;
+}
+
 /**
  * annotation_skip: (skip)
  * @list: a pointer to take a list
