@@ -20,38 +20,6 @@
 
 #include "tester.h"
 
-/**
- * bug_141869_a:
- * @pid: arg
- *
- * http://bugzilla.gnome.org/show_bug.cgi?id=141869
- */
-void bug_141869_a (unsigned pid) {
-  /* just silence a compiler warning */
-  bug_481811((double)pid);
-}
-
-/**
- * bug_141869_b:
- * @pid: arg
- *
- * http://bugzilla.gnome.org/show_bug.cgi?id=141869
- */
-void bug_141869_b (signed pid) {
-}
-
-
-/**
- * bug_379466:
- * @pid: arg
- *
- * http://bugzilla.gnome.org/show_bug.cgi?id=379466
- */
-void bug_379466 (int
-  pid) {
-}
-
-
 /**
  * bug_380824:
  * @arg: arg
@@ -68,19 +36,6 @@ int bug_380824 (int arg) {
 }
 
 
-/**
- * bug_411739:
- *
- * http://bugzilla.gnome.org/show_bug.cgi?id=411739
- *
- * Returns: result
- */
-struct bug_411739_rettype *
-bug_411739 (void) {
-  return NULL;
-}
-
-
 /**
  * bug_419997:
  * @const_values: arg
@@ -112,18 +67,6 @@ G_CONST_RETURN gchar* G_CONST_RETURN * bug_471014 (void) {
   return NULL;
 }
 
-
-/**
- * bug_552602:
- *
- * http://bugzilla.gnome.org/show_bug.cgi?id=552602
- *
- * Returns: result
- */
-const char* const * bug_552602 (void) {
-  return NULL;
-}
-
 /**
  * bug_574654a:
  *
@@ -490,4 +433,4 @@ bug_749142 (void)
 void
 bug_783420 (int in, int *out)
 {
-}
\ No newline at end of file
+}
