@@ -13,17 +13,17 @@ echo "Running suite(s): gtk-doc-$suite";
 # tests
 # check missing section description
 # we can't just check for a missing "tester_nodocs" entry
-grep >/dev/null "tester_nodocs:Long_Description" $DOC_MODULE-undocumented.txt
+grep >/dev/null "tester_nodocs:long_description" $DOC_MODULE-undocumented.txt
 if test $? = 1 ; then failed=`expr $failed + 1`; fi
 tested=`expr $tested + 1`
 
 # check missing section long description
-grep >/dev/null "tester_nolongdesc:Long_Description" $DOC_MODULE-undocumented.txt
+grep >/dev/null "tester_nolongdesc:long_description" $DOC_MODULE-undocumented.txt
 if test $? = 1 ; then failed=`expr $failed + 1`; fi
 tested=`expr $tested + 1`
 
 # check missing section short description
-grep >/dev/null "tester_noshortdesc:Short_Description" $DOC_MODULE-undocumented.txt
+grep >/dev/null "tester_noshortdesc:short_description" $DOC_MODULE-undocumented.txt
 if test $? = 1 ; then failed=`expr $failed + 1`; fi
 tested=`expr $tested + 1`
 
