@@ -56,17 +56,19 @@ if test $nok -gt 0 ; then failed=`expr $failed + 1`; fi
 tested=`expr $tested + 1`
 
 
-# check validity of generated sgml files
-nok=0
-for file in $dir/$suite/docs/xml/*.sgml; do
-  xmllint --noout --noent $file
-  if test $? != 0 ; then
-    echo 1>&2 "sgml validity check failed for $file"
-    nok=`expr $nok + 1`;
-  fi
-done
-if test $nok -gt 0 ; then failed=`expr $failed + 1`; fi
-tested=`expr $tested + 1`
+# check validity of generated sgml files (if any)
+if ls $dir/$suite/docs/xml/*.sgml 1> /dev/null 2>&1; then
+  nok=0
+  for file in $dir/$suite/docs/xml/*.sgml; do
+    xmllint --noout --noent $file
+    if test $? != 0 ; then
+      echo 1>&2 "sgml validity check failed for $file"
+      nok=`expr $nok + 1`;
+    fi
+  done
+  if test $nok -gt 0 ; then failed=`expr $failed + 1`; fi
+  tested=`expr $tested + 1`
+fi
 
 # check validity of devhelp2 files
 nok=0
@@ -80,13 +82,15 @@ done
 if test $nok -gt 0 ; then failed=`expr $failed + 1`; fi
 tested=`expr $tested + 1`
 
-# check that log files have only one line (the command)
-# discard references to launchapd bugs
 nok=0
-DISCARD_PATTERN='Please fix https://bugs.launchpad.net/ubuntu/+source/gtk-doc/+bug/[0-9]* . For now run:
+# check that log files have only one line (the command)
+# - discard references to launchapd bugs
+# - discard errors for missing optional tools.
+DISCARD_PATTERN1='Please fix https://bugs.launchpad.net/ubuntu/+source/gtk-doc/+bug/[0-9]* . For now run:
 gunzip .*.gz
 
 '
+DISCARD_PATTERN2='dblatex or fop must be installed.'
 for file in $dir/$suite/docs/gtkdoc-*.log; do
   # skip this in verbose mode as we'll have more text
   if test "x${V}" = "x1"; then
@@ -110,7 +114,8 @@ for file in $dir/$suite/docs/gtkdoc-*.log; do
     ;;
   esac
 
-  lines=`grep -v -x -G -e "$DISCARD_PATTERN" $file | wc -l | cut -d' ' -f1`
+  # count expected lines
+  lines=`grep -v -x -G -e "$DISCARD_PATTERN1" $file | grep -v -x -G -e "$DISCARD_PATTERN2" | wc -l | cut -d' ' -f1`
   if test $lines -gt $expected_lines; then
     echo 1>&2 "expected no more than $expected_lines log line in $file, but got $lines"
     nok=`expr $nok + 1`;
